;-- HbmMP85Test (Tests fuer HBM-"Verstaerker" MP85 mit CANopen) ------------
; Autor: Wolfgang Buescher     Stand: 2011-07-20
;   Ablage des Originals: //WoBu5/c/CBproj/CanTest/Programs/HbmMP85Test.txt .
;   Simuliertes Object-Dictionary laden aus Datei:
;      <WoBu5> C:\Programme\CanTestW\Objects\hbm_vis_sim.dic
;   Mit Test der Funktionalitaet der Firmware nach DS301 V4.0 (!!!)
;   Auch fuer andere Geraete mit "DS 301  V4.0" geeignet, z.B. Terminals, E/A-Module .
; 

;-- Einstellung Variablen ---------------------------------------------------
@print N,J,P,S,T,Y,L  ; Test: press F2 with cursor in this line to check settings !
@N=10  ; Node-ID (==DIP-Schalter-Adresse, beim Terminal: Systemmenue)
@J=384+N   ; Identifier f. PDO EA->Master (Inputs,Lenze: 767+N, DS401:384+N)
@P=512+N   ; Identifier f. PDO Master->EA (Outputs,Lenze: 768+N, DS401:512+N)
@S=1536+N  ; SDO(rx) des E/A-Master-Moduls
@T=1408+N  ; SDO(tx) des E/A-Master-Moduls (!)
@Y= 1      ; Option: 0=ohne SYNC  1=mit SYNC
@L= 1      ; Option: 0=Laufl.aus  1=Laufl.an
@sdo.node(N)    ; Node-ID fuer SDO-Zugriffe mit sdo.read() und sdo.write()
@print "NODE-ID fuer SDO-Zugriffe = %ld", sdo.node

;@set_scope.timing 20
;@set_scope(5).range 19800 20200
;@set_scope(5).range -32768 32767

;----- Simulation eines MP85(!) vorbereiten (Ersatz fr Messumformer) ----
; Dazu muss vorher MANUELL das CANopen-OD eines MP85 per EDS-File
; in das in den CAN-Tester integrierte "eigene" OD geladen werden !
; Im Hauptmen: Tools -> Lokales CANopen Objekt-VZ -> Laden .
; Einige Objekte im EIGENEN Objekt-Dictionary modifizieren:
obd.w($1008.0,STR,"Mein Name ist Hase")



;----- Communication Profile (DS301 V4) -------------------
@print ""
@print "------ Reading Communication Profile .. ------"
@print "DevType=$%08lX",sdo.r($1000.0)
@print "ErrorReg=$%08lX",sdo.r($1001.0)
@print "ManufStatusReg=$%08lX",sdo.r($1002.0)
@print "PreDefErrorFld=$%08lX",sdo.r($1003.0)
@print "NumberOftotalPDOs=$%08lX",sdo.r($1004.0)
@print "NumberOfsynchPDOs=$%08lX",sdo.r($1004.1)
@print "NumberOfasyncPDOs=$%08lX",sdo.r($1004.2)
@print "CobID_Sync=$%08lX",sdo.r($1005.0)
@print "CommCyclePer=%ld",sdo.r($1006.0)
@print sdo.r($1008.0)   ; device name (as "visible string")
@print sdo.r($1009.0)   ; hardware version ("visible string")
@print sdo.r($100A.0)   ; software version ("visible string")
@print "GuardTime=$%08lX",sdo.r($100C.0)
@print "LifeTimeFactor=$%08lX",sdo.r($100D.0)
@print "CobIdTimestamp=$%08lX",sdo.r($1012.0)
@print "CobIdEmergency=$%08lX",sdo.r($1014.0)
@print "EmgcyInhibitTi=$%08lX",sdo.r($1015.0)
@print "ProducerHeartb=$%08lX",sdo.r($1017.0)
@print "IdentityObjectSubis=%08lX",sdo.r($1018.0)
@print "RPDO1_CommPar=$%08lX",sdo.r($1400.0)
@print "RPDO1_CobID=$%08lX",sdo.r($1400.1)
@print "RPDO1_TranType=$%08lX",sdo.r($1400.2)
@print "RPDO1_InhiTime=$%08lX",sdo.r($1400.3)
@print "RPDO1_CMS_prio=$%08lX",sdo.r($1400.4)
@print "RPDO1_EvtTimer=$%08lX",sdo.r($1400.5)
@print "RPDO2_CommPar=$%08lX",sdo.r($1401.0)
@print "RPDO2_CobID=$%08lX",sdo.r($1401.1)
@print "RPDO2_TranType=$%08lX",sdo.r($1401.2)
@print "RPDO2_InhiTime=$%08lX",sdo.r($1401.3)
@print "RPDO2_CMS_prio=$%08lX",sdo.r($1401.4)
@print "RPDO2_EvtTimer=$%08lX",sdo.r($1401.5)


@stop   ; STOP READING --- some write accesses follow,
        ; only intended for the software developer
        ; using single-step execution of this command file !

@sdo.w ($1005.0,U32,$40000080) ; configure SYNC-Object
@sdo.w ($1006.0,U32,20000)  ; set comm-cycle to 20000us
@sdo.w($1010.1, U32, 0x65766173) ; "save" ALL PARAMETERS (!)
@sdo.w($1011.1, U32, 0x64616F6C) ; "load" DEFAULT PARAMETERS (!)
@print "TimeStampObj=$%08lX",sdo.r($1012.0)
@print "EmergcyObject=$%08lX",sdo.r($1014.0)
@print "EmergencyInhib=%ld",sdo.r($1015.0)
@print "HeartbeatConsumers=%ld",sdo.r($1016.0)
@print "HeartbeatConsumer1=$%08lX",sdo.r($1016.1)
@print "HeartbeatConsumer2=$%08lX",sdo.r($1016.2)
@print "HeartbeatProducerTime=%ld ms",sdo.r($1017.0)
@print "Vendor ID=$%08lX",sdo.r($1018.1) ; MKT=0x004D4B54
@print "Prod.Code=$%08lX",sdo.r($1018.2) ; HBM: Funktioniert nicht !
@print "RevisionN=$%08lX",sdo.r($1018.3)
@print "SerialNr.=$%08lX",sdo.r($1018.4)

; Configure 1st SDO CLIENT (in the CANopen node; terminal; I/O; etc)
@print "SDOC1_IDC2S=$%08lX",sdo.r($1280.1)  ; -> $0600+X
@print "SDOC1_IDS2C=$%08lX",sdo.r($1280.2)  ; -> $0580+X
@print "SDOC1_NODE=%ld",sdo.r($1280.3)      ; ->       X(dezimal)
; ex: @sdo.w($1280.3, I32, #10)  ; set node ID of the client's SERVER(!)
;  Trying to configure the 1st SDO CLIENT :
@sdo.w($1280.1, I32, 0x0600 + #10) ; COB-ID Client->Server
@sdo.w($1280.2, I32, 0x0580 + #10) ; COB-ID Server->Client
@sdo.w($1280.3, I8, #10)           ; Node-ID of the client's SERVER
@sdo.w($1010.1, U32, 0x65766173)   ; "save" ALL PARAMETERS (slow!)
000 81 00  ; "Reset all Nodes" -> All SDO-Client-Params LOST ?!?! 
           ;  Too bad to be true, but SYS TEC thought this is ok.
000 01 00   ; "Start Remode Node" -> no reaction (because cfg lost)




;-------------- "transmit" - PDO (analoge und digitale Inputs) -------------
   ;
   ; 0x1800 = 1st TPDO Communication parameter
@print "TxPDO1CommPar.Sub0=%ld",sdo.r($1800.0)
@print "TxPDO1CommPar.CobID=$%08lX",sdo.r($1800.1) ; PDO ID of 1st TPDO    ?
@print "TxPDO1CommPar.TrTyp= %03ld",sdo.r($1800.2) ; PDO transmission type ?
 ; (tr.type: 255=event  1..240=sync,  253=on RTR    [DS301 10-6] )
@sdo.w ($1800.2, U8, 255)                          ; set PDO transmission type
@print "TxPDO1CommPar.Inhib= %05ld",sdo.r($1800.3) ; PDO inhibit time      ?
@print "TxPDO1CommPar.CMSpr= %03ld",sdo.r($1800.4) ; PDO CMS priority (no) ?
@print "TxPDO1CommPar.EvTim= %05ld",sdo.r($1800.5) ; PDO event timer [ms]  ?
@sdo.w ($1800.5, U16, 1000) ; set PDO event timer [ms] (-> TXcycle if TxType==255)
@sdo.w ($1800.5, U16, 0) ; 0 = PDO event timer off

   ;
   ; 0x1A00 = 1st TPDO Mapping parameter
@print "TxPDO1Mapping.NrObj= %05ld",sdo.r($1A00.0) ; Number of mapped objs ?
@print "TxPDO1Mapping.Obj.1=$%08lX",sdo.r($1A00.1) ; First mapped object   ?
@print "TxPDO1Mapping.Obj.2=$%08lX",sdo.r($1A00.2) ; 2nd   mapped object   ?
@print "TxPDO1Mapping.Obj.3=$%08lX",sdo.r($1A00.3) ; 3rd   mapped object   ?
@print "TxPDO1Mapping.Obj.4=$%08lX",sdo.r($1A00.4) ; 4th   mapped object   ?
@stop  ; A "user-defined" PDO-Mapping for the 1st Transmit - PDO:
@sdo.w($1A00.0, U32, 4)  ; Map X objects into TPDO1
@sdo.w($1A00.1, U32, 0x60000108) ; 1st mapped object: 8 digital inputs
@sdo.w($1A00.2, U32, 0x60000208) ; 2nd mapped object: more digital inputs(?)
@sdo.w($1010.1, U32, 0x65766173) ; SAVE ALL PARAMETERS (!)

;-------------- "receive" - PDO (analoge+digitale Outputs) -----------------
   ;
   ; 0x1400 = 1st RPDO Communication parameter
@print "RxPDO1CommPar.Sub0=%ld",sdo.r($1400.0)
@print "RxPDO1CommPar.CobID=$%08lX",sdo.r($1400.1) ; PDO ID of 1st RPDO    ?
@print "RxPDO1CommPar.TrTyp= %03ld",sdo.r($1400.2) ; PDO transmission type ?
 ; (tr.type: 255=event  1..240=sync,  253=on RTR    [DS301 10-6] )
@print "RxPDO1CommPar.Inhib= %05ld",sdo.r($1400.3) ; PDO inhibit time      ?
@print "RxPDO1CommPar.CMSpr= %03ld",sdo.r($1400.4) ; PDO CMS priority (no) ?
@print "RxPDO1CommPar.EvTim= %05ld",sdo.r($1400.5) ; PDO event timer [ms]  ?
   ;
   ; 0x1600 = 1st RPDO Mapping parameter
@print "RxPDO1Mapping.NrObj= %05ld",sdo.r($1600.0) ; Number of mapped objs ?
@print "RxPDO1Mapping.Obj.1=$%08lX",sdo.r($1600.1) ; First mapped object   ?
@print "RxPDO1Mapping.Obj.2=$%08lX",sdo.r($1600.2) ; 2nd   mapped object   ?
@print "RxPDO1Mapping.Obj.3=$%08lX",sdo.r($1600.3) ; 3rd   mapped object   ?
@print "RxPDO1Mapping.Obj.4=$%08lX",sdo.r($1600.4) ; 4th   mapped object   ?

;----------- Objekte aus device profile (hier: E/A-Modul mit DS401) -------
@print "NR_DIGIN= %ld * 8bit",sdo.r($6000.0)   ; number of digital inputs
@print "ST_DIGIN= $%02lX",sdo.r($6000.1)   ; state of first 8 digital inputs
@print "NR_DIGOUT=%ld * 8bit",sdo.r($6200.0)   ; number of digital outputs
@print "ST_DIGOUT=$%02lX",sdo.r($6200.1)   ; state of first 8 digital outputs
@print "FAULT_MOD=%ld",sdo.r($6206.1) ; read fault mode for 8 outputs
@print "NR_AN_IN =%ld",sdo.r($6401.0) ; read number of analog Inputs
@print "AIN0     =%ld",sdo.r($6401.1) ; read 1st 16-bit-analog Input
@print "NR_AN_OUT=%ld",sdo.r($6411.0) ; read number of analog Outputs (?!)

;----------- Objekte aus device profile (hier: HBM-Messeverstrker) -------
@print "Messw_Xi= %ld",sdo.r($2000.1,I32) ; "Brutto Messwert x" (int32)
@print "Messw_Xf= %1g",sdo.r($3000.1,F32) ; "Bruttomesswert x"  (FLOAT32)
@print "Messw_Yi= %ld",sdo.r($2000.2,I32) ; "Brutto Messwert x" (int32)
@print "Messw_Yf= %lg",sdo.r($3000.2,F32) ; "Bruttomesswert x"  (FLOAT32)



@stop
(S) 22 $1010.w 1 "save"   ; "save" Parameters ($1010.1) in EEPROM

;----------- "user specific", Codenummern aus "Systemhandbuch" --------------
@print "CurrPage=%ld",sdo.r($5105.1)     ; Nummer der aktuellen Anzeigeseite
@sdo.w($5105.1,U16, 0 )    ; switch to page nr 0
@sdo.w($5105.1,U16, 1 )    ; switch to page nr 1
@sdo.w($5105.1,U16, 2 )    ; switch to page nr 2
@sdo.w($5105.1,U16, 1000 ) ; switch to page nr 1000 (must be rejected w. ERROR CODE)
@print "NrInt16Vars=%ld",sdo.r($4003.0)   ; Anzahl der 16-Bit-INTEGER-Variablen
@print "NrInt32Vars=%ld",sdo.r($4004.0)   ; Anzahl der 32-Bit-INTEGER-Variablen
@print "NrUns8Vars=%ld", sdo.r($4005.0)   ; Anzahl der  8-Bit-UNSIGNED-Variablen
@print "NrUns16Vars=%ld",sdo.r($4006.0)   ; Anzahl der 16-Bit-UNSIGNED-Variablen
@print "NrUns32Vars=%ld",sdo.r($4007.0)   ; Anzahl der 32-Bit-UNSIGNED-Variablen
@print "NrFlt32Vars=%ld",sdo.r($4008.0)   ; Anzahl der 32-Bit-FLOAT-Variablen
@print "NrStringVars=%ld",sdo.r($4009.0)  ; Anzahl der 32-Bit-FLOAT-Variablen
@print sdo.r($4009.1)   ; read first "string" var in manufacturer specific part
@sdo.w($4009.1,STR,"Hello WoBu !")  ; set first "string"-var through SDO channel



@print "Int16Var[1]=%ld",sdo.r($4003.1)   ; erste 16-Bit-INTEGER-Variable lesen
@sdo.w($4003.1,I16, 12345)                ; erste 16-Bit-INTEGER-Variable setzen
@print "Int32Var[10]=%ld",sdo.r($4004.10) ; zehnte 32-Bit-INTEGER-Variable lesen
@sdo.w($4004.10,I32, -1234567890)         ; zehnte 32-Bit-INTEGER-Variable setzen


@stop ; end diagnostic READ section

(S) 22 $1010.w 1 "save"           ; "save" Parameters ($1010.1) in EEPROM

;--- Aufruf spezieller Test-und Einstellprozeduren (Codenummern 2048...) -----

;====== Zustandsuebergaenge des CANopen-Knotens (Test) =======================
($700+N)rXX ; Anforderung Node-Guard-Status mit RTR (nur mit NEUEM PCAN-USB!)
000 01 00   ; "Start Remode Node" ("Node Start Protocol" in DS203-2 p.7)
000 02 00   ; "Stop Remote Node"
000 03 00   ; "Disconnect Remote Node"       (DS203-2 p.8)
000 80 00   ; "Enter Preoperational State"
000 81 00   ; Operational -> Initialisation (d.h. Reset)
#128 0      ; Sync-Telegramm
@stop
@reset  ; reset time, counters and warn-flags of the DOS-CAN-Tester

LaufLicht:  ;====== Ansteuern der 8 digitalen I/Os per PDO =======
 @gosub ExtBoot ; "Extended Bootup" wichtig fuer Node-Guarding.
 000 01 00  ; Modul von Pre-Operational -> Operational schalten, PDOs aktiv.
Lauf1:      ; Auch fuer LENZE mit Master/Slave - Kaskadierung (oh Graus...)
 (P) 01 C0 07 F0
 (P) 02 60 0E 78
 (P) 04 30 1C 3C
 (P) 08 18 38 1E
 (P) 10 0C 70 0F
 (P) 20 06 E0 87
 (P) 40 03 C1 C3
 (P) 80 01 83 E1
 ($700+N)rXX ; Anforderung Node-Guard-Status via RTR
@goto Lauf1

ExtBoot:     ; Unterprogramm "Bootup MIT Nodeguarding und SYNC"
000 80 00    ; Enter Pre-Operational State
000 03 00    ; Disconnect -> Antwort "#2022"=identify node
($700+N)rXX  ; Anforderung Node-Guard-Status
(#2026) 04 (N) 00 00 00 00 00 00  ; Connect Remote Node ("Teil 1")
(#2026) 06 00  FF 00 00 00 00 00  ; "Identify Remote Nodes" (!!??!!)
(#2026) 02 (N) (N) 07 00 10 02 02 ; Node Guarding Vorbereitung [203-2p4],"T.2"
(#2026) 03 (N) 01 00 00 00 00 00  ; Prepare Remote Node
@return  ; Ende "ExtBoot"


; Test des IM CAN-TESTER EINGEBAUTEN(!) CANopen-ODs, mit Schreib- und Lesezugriffen:
@print "DeviceName =",obd.r($1008.0) ; "manufacturer device name" (wird auch vom DT85 ausgelesen und angezeigt)
obd.w($1008.0,STR,"Mein Name ist Hase")
@print "CanNodeName=",obd.r($100B.0)



; <EOF>
